/* Copyright (c) 2007 by IBM Corporation */

dojo.provide("zero.form.widget.field.FormFieldBaseMixin");


dojo.require("zero.form.common.components.NlsMixin");

dojo.declare( "zero.form.widget.field.FormFieldBaseMixin", 
		[zero.form.common.components.NlsMixin
		],
    {
 		handlers: null,
		shortName: "",
		dataId: "",
		title: "",
		hint: "",
		required: false,
		description: "",
		defaultValue: null,
		type: "",
		eventFuncPrefix: "__",
		supportedEvents: null,
		
		postCreate: function()
		{
			this.supportedEvents = ["onChange","onMouseOut","onMouseOver","onClick"];
		},
		
		startup: function()
		{
			this.setContent(this.contentJson);	
			this.buildHandlers();
			this.updateHandlers(); // Added by Pablo need to be reviewd by Yoav.
			this.initEventsCallbacks();
		},

		/**
		 * Fills form widget's properties according to the given json
		 * @param {Object} pJson json representing this form widget content
		 */
		setContent: function(pJson)
		{
			if (pJson === undefined)
			{
				return;
			}
			this.setShortName(pJson.shortName);
			this.setTitle(pJson.title);
			this.setHint(pJson.hint);
			this.setIsRequired(pJson.required);
			this.setDescription(pJson.description);
			this.setDataId(pJson.dataId);
			this.setDefaultValue(pJson.default_value);
			this.setIsImmutable(pJson["immutable"]);
			this.setAdditionalProperties(pJson);
		},
		
		/**
		 * Returns the json representation of this form widget
		 * @return {Object} json representing this form widget
		 */
		getContent: function()
		{
			var result = {
				shortName: this.shortName,
				type: this.type,
				title: this.title || "Title",
				hint: this.hint || "",
				required: this.required || false,
				description: this.description || "",
				dataId: this.dataId || "",
				"immutable":this["immutable"] || false,
				default_value: this.defaultValue || ""
			};
			
			this.getEventsContent(result);
			this.getAdditionalProperties(result);
			
			return result;
		},
		
		getEventsContent: function(result)
		{
			var eventName;
			for(var i=0,l=this.supportedEvents.length;i<l;i++)
			{
				eventName = this.supportedEvents[i];
				result[eventName] =  this.contentJson[eventName] || "";
			}
		},

		// getters and setters for basic form widget's properties
		getShortName: function()
		{
			return this.shortName;
		},

		setShortName: function(pShortName)
		{
			this.shortName = pShortName;
		},

		getTitle: function()
		{
			return this.title;
		},

		setTitle: function(pTitle)
		{
			this.title = pTitle;
		},

		getHint: function()
		{
			return this.hint;
		},

		setHint: function(pHint)
		{
			this.hint = pHint;
		},

		isRequired: function()
		{
			return this.required;
		},

		setIsRequired: function(pIsRequired)
		{
			this.required = pIsRequired;
		},

		isImmutable: function()
		{
			return this["immutable"];
		},

		setIsImmutable: function(pIsImmutable)
		{
			this["immutable"] = pIsImmutable;
		},

		getDescription: function()
		{
			return this.description;
		},

		setDescription: function(pDescription)
		{
			this.description = pDescription;
		},

		getDataId: function()
		{
			return this.dataId;
		},

		setDataId: function(pDataId)
		{
			this.dataId = pDataId;
		},

		getDefaultValue: function()
		{
			return this.defaultValue;
		},

		setDefaultValue: function(pDefaultValue)
		{
			this.defaultValue = pDefaultValue;
		},

		buildHandlers: function()
		{
			this.handlers = this.getHandlers();
			for(var i=0;i<this.handlers.length;i++)
				this.handlers[i].setWidget(this);			
		},
		
		getHandlers: function()
		{
			return [];
		},
		
		updateHandlers: function()
		{
			if(!this.handlers)
				return;
			for(var i=0;i<this.handlers.length;i++)
				this.handlers[i].update();
		},
		
		getValueForDB: function()
        {
            return this.getValue();
        },
		
		setAdditionalProperties: function(pJson) {},
		getAdditionalProperties: function(pJson) {},
		
		onMouseOut: function(){},
		onChanged: function(){},
		onMouseOver: function(){},
		onClick: function(){},
		
		initEvent: function(eventName)
		{
			if(!this.blockEventsInitiation && this.contentJson &&  this.contentJson[eventName] && (typeof this.contentJson[eventName] === 'string'))
			{
				var actualFuncName = this.eventFuncPrefix+eventName;
				this[actualFuncName] = new Function(this.contentJson[eventName]);
				dojo.connect(this,eventName,this,actualFuncName);
			}
		},
		
		initEventsCallbacks: function()
		{
			for(var i=0,l=this.supportedEvents.length;i<l;i++)
			{
				this.initEvent(this.supportedEvents[i]);
			}
		}
	
    });
